// Tasks.cpp: implementation of the CTasks class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#include "OpCode.h"
#include "../EmEngine.h"
#include "../../OtherFunctions.h"

//////////////////////////////////////////////////////////////////////
// Client OpCodes
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// HELLO opcode
bool COpCode_HELLO::ProcessForClient(CEmEngine& stEngine, CEmClient* pClient)
{
	ASSERT(pClient); // this opcode is processed only in the main context

//	// here we should grab all user's info
	COpCode_HELLOANSWER stMsg;
	md4cpy(stMsg._UserHash, stEngine.GetUserHash());
	stMsg._ClientAddr._IPAddr = stEngine.GetClientID();
	stMsg._ClientAddr._Port = stEngine.GetPort();
	stMsg._UserName.m_strValue = stEngine.GetUserNick();
	stMsg._UserName.m_bValid = TRUE;
	stMsg._ClientSoftVersion.m_dwValue = EDONKEYVERSION;
	stMsg._ClientSoftVersion.m_bValid = TRUE;
	if(stEngine.IsConnected())
	{
		stMsg._ServerAddr._IPAddr = stEngine.GetConnectedState()->nAddr;
		stMsg._ServerAddr._Port = stEngine.GetConnectedState()->nPort;
	}
	else
	{
		stMsg._ServerAddr._IPAddr = 0;
		stMsg._ServerAddr._Port = 0;
	}

	stEngine.SendOpCode(m_hSocket, stMsg, pClient);

	if (1/*emule protocol supported*/)
	{
		// send emuleinfo
	}

	return true;
}

//////////////////////////////////////////////////////////////////////
// HELLO ANSWER
bool COpCode_HELLOANSWER::ProcessForClient(CEmEngine& stEngine, CEmClient*)
{
	return true;
}

bool COpCode_EMULEINFO::ProcessForClient(CEmEngine& stEngine, CEmClient* pClient)
{
	// send answer

	return true;
}

bool COpCode_EMULEINFOANSWER::ProcessForClient(CEmEngine& stEngine, CEmClient* pClient)
{
	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// OFFER FILES
bool COpCode_OFFERFILES::ProcessForClient(CEmEngine& stEngine, CEmClient*)
{
	return true;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// Server OpCodes
//////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// SERVER MESSAGE
bool COpCode_SERVERMESSAGE::ProcessForClient(CEmEngine& stEngine, CEmClient* pClient)
{
	CEmClient_Server* pSrv = reinterpret_cast<CEmClient_Server*>(pClient);

	// Increase message count
	m_nMsgCount = pSrv->m_nMsgCount++;

	TRACE("MESSAGE FROM SERVER: %s\n", _Msg);

//	m_Server._Addr = pSrv->m_dwAddr;
//	m_Server._Port = pSrv->m_uPort;

//	stEngine.PushToUI(this);

	return false; // Always return false when going to GUI, otherwise this object will be destroyed!
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// SERVER MESSAGE in GUI
/*bool COpCode_SERVERMESSAGE::ProcessForUI(CEmEngine&)
{
	// Print message
	g_eMuleApp.m_pdlgEmule->AddServerMessageLine(_Msg);

	// Check for server version
	if ((m_nMsgCount < 2) && (_Msg.Left(14).CompareNoCase(_T("server version")) == 0))
	{
		CString strVer = _Msg.Mid(14);
		strVer.Trim();
		strVer = strVer.Left(64); // Truncate string to avoid misuse by servers in showing ads
		int n = strVer.FindOneOf("\x09\x0A\x0D");
		if (n >= 0)
			strVer = strVer.Left(n);
		CServer* pServer = g_eMuleApp.m_pServerList->GetServerByIP(m_Server._Addr, m_Server._Port);
		if (pServer)
			pServer->SetVersion(strVer);
	}

	// Check for dynamic IP
	int nDynIP = _Msg.Find(_T("[emDynIP: "));
	int nCloseBracket = _Msg.Find(_T("]"));
	if (nDynIP != -1 && nCloseBracket != -1 && nDynIP < nCloseBracket)
	{
		CString sDynIP = _Msg.Mid(nDynIP + 10, nCloseBracket - (nDynIP + 10));
		sDynIP.Trim(_T(" "));
		if (sDynIP.GetLength() && sDynIP.GetLength() < 51)
		{
			CServer* pServer = g_eMuleApp.m_pServerList->GetServerByIP(m_Server._Addr, m_Server._Port);
			if (pServer)
			{
				pServer->SetDynIP(sDynIP.GetBuffer());
				g_eMuleApp.m_pdlgEmule->m_wndServer.m_ctlServerList.RefreshServer(*pServer);
			}
		}
	}

	return true;
}*/
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// ID CHANGE
bool COpCode_IDCHANGE::ProcessForClient(CEmEngine& stEngine, CEmClient* pClient)
{
	ASSERT(pClient); // this opcode is processed only in the main context

	// TODO: Smart ID checks, notifiers

	// Disconnect from previous connected server, if any
	if(stEngine.GetServerSocket() != pClient->m_hSocket &&
		stEngine.GetServerSocket())
		stEngine.DisconnectFromServer(stEngine.GetServerSocket());

	// Set server connection info
	stEngine.ConnectedTo(reinterpret_cast<CEmClient_Server*>(pClient));
	stEngine.SetClientID(_NewClientID);
	stEngine.SetConnectionState(SERVER_CONNECTED);

/*
	if (g_eMuleApp.m_pGlobPrefs->GetAddServersFromConn())
	{
		Packet* packet = new Packet(OP_GETSERVERLIST);
		g_eMuleApp.m_pUploadQueue->AddUpDataOverheadServer(packet->size);
		SendPacket(packet,true);
	}
	if(g_eMuleApp.m_pGlobPrefs->RestartWaiting())
		g_eMuleApp.m_pdlgEmule->m_wndTransfer.m_ctlDownloadList.RestartWaitingDownloads();
 */

	return true;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// SEARCH RESULT
bool COpCode_SEARCHRESULT::ProcessForClient(CEmEngine& stEngine, CEmClient* pClient)
{
	ASSERT(pClient); // this opcode is processed only in the main context

	return true;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// FOUND SOURCES
bool COpCode_FOUNDSOURCES::ProcessForClient(CEmEngine& stEngine, CEmClient* pClient)
{
	ASSERT(pClient); // this opcode is processed only in the main context

	// Checking and adding sources (logic moved from PartFile.cpp, later should make this separate task),
	// later rebuild (?) PartFile to separate task processor
/*	if (CPartFile* pFile = g_eMuleApp.m_pDownloadQueue->GetFileByID(_Hash))
	{
		if(  !(pFile->GetStatus() == PS_STOPPED
			|| pFile->GetStatus() == PS_COMPLETE
			|| pFile->GetStatus() == PS_COMPLETING))
		{
			byte debug_lowiddropped = 0;
			byte debug_possiblesources = 0;

			//	If there is needed place for more sources, just remove NNS
			if ( pFile->GetSourceCount() + _Sources.GetSize() > g_eMuleApp.m_pGlobPrefs->GetMaxSourcePerFile()
				&& ((pFile->GetLastPurgeTime() == 0) || (::GetTickCount() - pFile->GetLastPurgeTime()) > PURGE_TIME) )
			{
				pFile->RemoveNoNeededSources(pFile->GetSourceCount() + _Sources.GetSize() - g_eMuleApp.m_pGlobPrefs->GetMaxSourcePerFile());
			}

			for (int i = 0; i < _Sources.GetSize(); i++)
			{
				//	check first if we are this source
				if (stEngine.IsConnected() && stEngine.IsLowID())
				{
					if ((stEngine.GetClientID() == _Sources[i]._Addr) )
// should fix that later, when add FullIP workout
//						&& inet_addr(g_eMuleApp.m_pServerConnect->GetCurrentServer()->GetFullIP()) == pClient->m_dwAddr)
						continue;
				}
				else if (stEngine.GetClientID() == _Sources[i]._Addr)
					continue;
				else if (_Sources[i]._Addr < 0x1000000 && !stEngine.IsLocalServer(pClient->m_dwAddr, pClient->m_uPort))
					continue;

				if (g_eMuleApp.m_pGlobPrefs->GetMaxSourcePerFile() > pFile->GetSourceCount())
				{
					CUpDownClient* pNewSource = g_eMuleApp.m_pDownloadQueue->CheckAndAddSource(pFile, _Sources[i]._Addr, _Sources[i]._Port, pClient->m_dwAddr, pClient->m_uPort);
					if (pNewSource)
						pNewSource->SetUserName(GetResString(IDS_SERVER_SOURCE));
				}

			}
		}
	}*/
	return true;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// SERVER STATUS
bool COpCode_SERVERSTATUS::ProcessForClient(CEmEngine& stEngine, CEmClient* pClient)
{
	ASSERT(pClient); // this opcode is processed only in the main context

	// Set address and port
//	m_Server._Addr = pClient->m_dwAddr;
//	m_Server._Port = pClient->m_uPort;

//	stEngine.PushToUI(this);

	TRACE("SERVER STATUS: %u users, %u files\n", _NumberOfUsers, _NumberOfFiles);

	return false; // Always return false when going to GUI, otherwise this object will be destroyed!
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// SERVER STATUS in GUI
/*bool COpCode_SERVERSTATUS::ProcessForUI(CEmEngine&)
{
	CServer* pServer = g_eMuleApp.m_pServerList->GetServerByIP(m_Server._Addr, m_Server._Port);
	if (pServer)
	{
		pServer->SetUserCount(_NumberOfUsers); 
		pServer->SetFileCount(_NumberOfFiles);
		g_eMuleApp.m_pdlgEmule->ShowUserCount(_NumberOfUsers, _NumberOfFiles);
		g_eMuleApp.m_pdlgEmule->m_wndServer.m_ctlServerList.RefreshServer(*pServer);
	}

	return true;
}*/
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// SERVER IDENTification
bool COpCode_SERVERIDENT::ProcessForClient(CEmEngine& stEngine, CEmClient* pClient)
{
	ASSERT(pClient); // this opcode is processed only in the main context

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// SERVER LIST
bool COpCode_SERVERLIST::ProcessForClient(CEmEngine& stEngine, CEmClient* pClient)
{
	ASSERT(pClient); // this opcode is processed only in the main context

	return true;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	CALLBACKREQUEST is a Client => Server message and should never be
//		received by a client.
bool COpCode_CALLBACKREQUEST::ProcessForClient(CEmEngine& stEngine, CEmClient *pClient)
{
	return true;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	CALLBACKFAIL (if it is ever received by a client) is ignored by
//		eMulePlus.
bool COpCode_CALLBACKFAIL::ProcessForClient(CEmEngine &stEngine, CEmClient *pClient)
{
	return true;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// CALLBACK REQUESTED
bool COpCode_CALLBACKREQUESTED::ProcessForClient(CEmEngine& stEngine, CEmClient* )
{
	CTask_SayHelloToPeer   *pSayHelloTask = new CTask_SayHelloToPeer();
	CTask_Connect		   *pTask = new CTask_Connect(_ClientAddr._IPAddr, _ClientAddr._Port, T_CLIENT_PEER, pSayHelloTask);

	stEngine.PushToSocketsTP(pTask);

	AddDebugLogLine("Callback requested");

	return true;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// LOGIN REQUEST
bool COpCode_LOGINREQUEST::ProcessForClient(CEmEngine& stEngine, CEmClient* )
{
	return true;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
