//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//	FriendListCtrl.cpp : implementation file
//

#include "stdafx.h"
#include "emule.h"
#include "FriendListCtrl.h"
#include "Friend.h"
#include ".\\Details\\ClientDetails.h"
#include "AddFriend.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#define new DEBUG_NEW
#endif

//	CFriendListCtrl

IMPLEMENT_DYNAMIC(CFriendListCtrl, CMuleListCtrl)
CFriendListCtrl::CFriendListCtrl()
{
}

CFriendListCtrl::~CFriendListCtrl()
{
}

BEGIN_MESSAGE_MAP(CFriendListCtrl, CMuleListCtrl)
	ON_NOTIFY_REFLECT(NM_RCLICK, OnNMRclick)
	ON_NOTIFY_REFLECT(NM_DBLCLK, OnDoubleclick)
	ON_WM_KEYDOWN()
	ON_NOTIFY_REFLECT(LVN_COLUMNCLICK, OnLvnColumnclick)
END_MESSAGE_MAP()

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	CFriendListCtrl message handlers
void CFriendListCtrl::Init()
{
	static const uint16 s_auIconResID[] =
	{
		IDI_FRIENDS1,
		IDI_FRIENDS2,
		IDI_FRIENDS3
	};

	SetExtendedStyle(LVS_EX_FULLROWSELECT);
	RECT rcWindow;
	GetWindowRect(&rcWindow);

	InsertColumn(0, _T(""), LVCFMT_LEFT, rcWindow.right - rcWindow.left - 4, 0);

	m_imageList.Create(16, 16, ILC_COLOR32 | ILC_MASK, 0, 10);
	m_imageList.SetBkColor(RGB(255, 255, 255));
	FillImgLstWith16x16Icons(&m_imageList, s_auIconResID, ELEMENT_COUNT(s_auIconResID));
	SetImageList(&m_imageList, LVSIL_SMALL);

	g_eMuleApp.m_pFriendList->SetWindow(this);
	g_eMuleApp.m_pFriendList->ShowFriends();
	m_bSortAscending = true;
	SetSortArrow(0, true);
	SortItems(SortProc, 0);
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CFriendListCtrl::Localize()
{
	CHeaderCtrl	*pHeaderCtrl = GetHeaderCtrl();
	CString		strRes = GetResString(IDS_QL_USERNAME);
	HDITEM		hdi;

	hdi.mask = HDI_TEXT;
	hdi.pszText = (LPSTR)strRes.GetString();
	pHeaderCtrl->SetItem(0, &hdi);
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CFriendListCtrl::AddFriend(CFriend* toadd)
{
	uint32 itemnr = GetItemCount();
	itemnr = InsertItem(LVIF_TEXT | LVIF_PARAM | LVIF_IMAGE, itemnr, toadd->m_strName.GetBuffer(), 0, 0, 1, (LPARAM)toadd);
	RefreshFriend(toadd);
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CFriendListCtrl::RemoveFriend(CFriend* toremove)
{
	LVFINDINFO find;
	find.flags = LVFI_PARAM;
	find.lParam = (LPARAM)toremove;
	sint32 result = FindItem(&find);
	if (result != ( -1))
		DeleteItem(result);
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CFriendListCtrl::RefreshFriend(CFriend *pFriend)
{
	if (!::IsWindow(GetSafeHwnd()))
		return;

	LVFINDINFO		find;

	find.flags = LVFI_PARAM;
	find.lParam = reinterpret_cast<LPARAM>(pFriend);

	sint32		iIndex = FindItem(&find);

	SetItemText(iIndex, 0, pFriend->m_strName);

	if (iIndex == -1)
		return;

	byte		iImageIndex;

	if (!pFriend->m_pLinkedClient)
		iImageIndex = 0;
#ifdef OLD_SOCKETS_ENABLED
	else if (pFriend->m_pLinkedClient->m_pRequestSocket && pFriend->m_pLinkedClient->m_pRequestSocket->IsConnected())
		iImageIndex = 2;
#endif //OLD_SOCKETS_ENABLED
	else
		iImageIndex = 1;

	SetItem(iIndex, 0, LVIF_IMAGE, 0, iImageIndex, 0, 0, 0, 0);
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CFriendListCtrl::OnDoubleclick(NMHDR *pNMHDR, LRESULT *pResult)
{
	EMULE_TRY

	int		iSelectionMark = GetSelectionMark();

	if ((iSelectionMark != -1) && (GetSelectedCount() > 0))
	{
		CFriend *cur_friend = reinterpret_cast<CFriend*>(GetItemData(iSelectionMark));
		if (g_eMuleApp.m_pGlobPrefs->GetDetailsOnClick() && cur_friend->m_pLinkedClient)
		{
			CClientDetails dialog(GetResString(IDS_CD_TITLE), cur_friend->m_pLinkedClient, this, 0);
			dialog.DoModal();
		}
	}

	EMULE_CATCH
	*pResult = 0;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CFriendListCtrl::OnNMRclick(NMHDR *pNMHDR, LRESULT *pResult)
{
	CTitleMenu	menuClient;
	POINT		point;

	::GetCursorPos(&point);

	menuClient.CreatePopupMenu();
	menuClient.AddMenuTitle(GetResString(IDS_FRIENDLIST));

	int			iSelectionMark = GetSelectionMark();
	bool		bSelected = ((iSelectionMark != -1) && (GetSelectedCount() > 0));

	if (bSelected)
	{
		CFriend		*cur_friend = reinterpret_cast<CFriend*>(GetItemData(iSelectionMark));

		menuClient.AppendMenu( MF_STRING | ((cur_friend->m_pLinkedClient) ? MF_ENABLED : MF_GRAYED),
			MP_DETAIL, GetResString(IDS_SHOWDETAILS) );
	}

	menuClient.AppendMenu(MF_STRING, MP_ADDFRIEND, GetResString(IDS_ADDAFRIEND));

	if (bSelected)
	{
		menuClient.AppendMenu(MF_STRING, MP_REMOVEFRIEND, GetResString(IDS_REMOVEFRIEND));
		menuClient.AppendMenu(MF_STRING, MP_MESSAGE, GetResString(IDS_SEND_MSG));
		menuClient.AppendMenu(MF_STRING, MP_SHOWLIST, GetResString(IDS_VIEWFILES));
	}
	menuClient.TrackPopupMenuEx(TPM_LEFTALIGN | TPM_RIGHTBUTTON, point.x, point.y, this, NULL);
	*pResult = 0;
//	Menu objects are destroyed in their destructor
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
BOOL CFriendListCtrl::OnCommand(WPARAM wParam, LPARAM lParam)
{
	int		iSelectionMark = GetSelectionMark();
	CFriend	*cur_friend = NULL;

	if (iSelectionMark != -1)
		cur_friend = reinterpret_cast<CFriend*>(GetItemData(iSelectionMark));
	switch (wParam)
	{
		case MP_MESSAGE:
		{
			if (cur_friend)
			{
				if (cur_friend->m_pLinkedClient)
					g_eMuleApp.m_pdlgEmule->m_wndChat.StartSession(cur_friend->m_pLinkedClient);
				else
				{
					CUpDownClient* chatclient = new CUpDownClient(cur_friend->m_nLastUsedPort, cur_friend->m_dwLastUsedIP, 0, 0, 0);
					chatclient->SetUserName(cur_friend->m_strName);
					if (g_eMuleApp.m_pClientList->AddClient(chatclient))
						g_eMuleApp.m_pdlgEmule->m_wndChat.StartSession(chatclient);
					else
						safe_delete(chatclient);
				}
			}
			break;
		}
		case MP_REMOVEFRIEND:
		{
			if (cur_friend)
				g_eMuleApp.m_pFriendList->RemoveFriend(cur_friend);
			break;
		}
		case MP_ADDFRIEND:
		{
			CAddFriend dialog2;
			dialog2.DoModal();
		}
		break;
		case MP_DETAIL:
		if (cur_friend && cur_friend->m_pLinkedClient)
		{
			CClientDetails dialog(GetResString(IDS_CD_TITLE), cur_friend->m_pLinkedClient, this, 0);
			dialog.DoModal();
		}
		break;
		case MP_SHOWLIST:
		{
			if (cur_friend)
			{
				if (cur_friend->m_pLinkedClient)
					cur_friend->m_pLinkedClient->RequestSharedFileList();
				else
				{
					CUpDownClient* newclient = new CUpDownClient(cur_friend->m_nLastUsedPort, cur_friend->m_dwLastUsedIP, 0, 0, 0);
					newclient->SetUserName(cur_friend->m_strName);
					//eklmn: check if client was inserted in client list
					if (g_eMuleApp.m_pClientList->AddClient(newclient))
						newclient->RequestSharedFileList();
					else
						safe_delete(newclient);
				}
			}
			break;
		}
	}
	return true;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CFriendListCtrl::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags)
{
	EMULE_TRY
	
	bool bHandled = false;

	if (nChar == VK_DELETE)
	{
		int		iSelectionMark = GetSelectionMark();
		CFriend	*pFriend = NULL;

		if (iSelectionMark != -1)
			pFriend = reinterpret_cast<CFriend*>(GetItemData(iSelectionMark));

		if( (pFriend != NULL) && ( !g_eMuleApp.m_pGlobPrefs->IsConfirmFriendDelEnabled() ||
			( MessageBox(GetResString(IDS_BACKUP_SURE), GetResString(IDS_REMOVEFRIEND),
			MB_ICONQUESTION | MB_YESNO) == IDYES ) ) )
		{
			g_eMuleApp.m_pFriendList->RemoveFriend(pFriend);
		}
		bHandled = true;
	}
	else if (nChar == VK_INSERT)
	{
		CAddFriend dialog2;
		dialog2.DoModal();

		bHandled = true;
	}

	if (!bHandled)
		CListCtrl::OnKeyDown(nChar, nRepCnt, nFlags);

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CFriendListCtrl::OnLvnColumnclick(NMHDR *pNMHDR, LRESULT *pResult)
{
	NM_LISTVIEW * pNMListView = (NM_LISTVIEW*)pNMHDR;
	m_bSortAscending = !m_bSortAscending;
	SetSortArrow(pNMListView->iSubItem, m_bSortAscending);
	SortItems(SortProc, pNMListView->iSubItem + ((m_bSortAscending) ? MLC_SORTASC : MLC_SORTDESC));
	*pResult = 0;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int CFriendListCtrl::SortProc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort)
{
	CFriend* item1 = (CFriend*)lParam1;
	CFriend* item2 = (CFriend*)lParam2;

	if (item1 == NULL || item2 == NULL)
		return 0;

	switch (LOWORD(lParamSort))
	{
		case 0:
			return _tcsicmp(item1->m_strName, item2->m_strName);
		case 0+MLC_SORTDESC:
			return _tcsicmp(item2->m_strName, item1->m_strName);
		default:
			return 0;
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////